/* modprobe.h: add or remove a module from the kernel, intelligently.
    Copyright (C) 2001  Rusty Russell.
    Copyright (C) 2002, 2003  Rusty Russell, IBM Corporation.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#define _GNU_SOURCE /* asprintf */

#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <dirent.h>
#include <limits.h>
#include <elf.h>
#include <getopt.h>
#include <fnmatch.h>
#include <asm/unistd.h>
#include <sys/wait.h>
#include <syslog.h>

#define streq(a,b) (strcmp((a),(b)) == 0)
#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#include "list.h"

extern long init_module(void *, unsigned long, const char *);
extern long delete_module(const char *, unsigned int);

struct module {
	struct list_head list;
	char *modname;
	char filename[0];
};

#ifndef MODULE_DIR
#define MODULE_DIR "/lib/modules"
#endif

typedef void (*errfn_t)(const char *fmt, ...);

extern int log;

void message(const char *prefix, const char *fmt, va_list *arglist);

void warn(const char *fmt, ...);

void fatal(const char *fmt, ...);

void grammar(const char *cmd, const char *filename, unsigned int line);

void *do_nofail(void *ptr, const char *file, int line, const char *expr);

#define NOFAIL(ptr)	do_nofail((ptr), __FILE__, __LINE__, #ptr)

void print_usage(const char *progname);

int fgetc_wrapped(FILE *file, unsigned int *linenum);

char *getline_wrapped(FILE *file, unsigned int *linenum);

struct module *find_module(const char *filename, struct list_head *list);

/* Convert filename to the module name.  Works if filename == modname, too. */
void filename2modname(char *modname, const char *filename);

int lock_file(const char *filename);

void unlock_file(int fd);

void add_module(char *filename, int namelen, struct list_head *list);

/* Compare len chars of a to b, with _ and - equivalent. */
int modname_equal(const char *a, const char *b, unsigned int len);

/* Fills in list of modules if this is the line we want. */
int add_modules_dep_line(char *line,
				const char *name,
				struct list_head *list);

void read_depends(const char *dirname,
			 const char *start_name,
			 int avoid_cache,
			 struct list_head *list);

/* We use error numbers in a loose translation... */
const char *insert_moderror(int err);

const char *remove_moderror(int err);

/* Is module in /proc/modules?  If so, fill in usecount if not NULL. 
   0 means no, 1 means yes, -1 means unknown.
 */
int module_in_kernel(const char *modname, unsigned int *usecount);

void replace_modname(struct module *module,
			    void *mem, unsigned long len,
			    const char *oldname, const char *newname);

void *get_section32(void *file,
			   unsigned long size,
			   const char *name,
			   unsigned long *secsize);

void *get_section64(void *file,
			   unsigned long size,
			   const char *name,
			   unsigned long *secsize);

int elf_ident(void *mod, unsigned long size);

void *get_section(void *file,
			 unsigned long size,
			 const char *name,
			 unsigned long *secsize);

void rename_module(struct module *module,
			  void *mod,
			  unsigned long len,
			  const char *newname);

/* Kernel told to ignore these sections if SHF_ALLOC not set. */
void invalidate_section32(void *mod, const char *secname);

void invalidate_section64(void *mod, const char *secname);

void strip_section(struct module *module,
			  void *mod,
			  unsigned long len,
			  const char *secname);

const char *next_string(const char *string, unsigned long *secsize);

void clear_magic(struct module *module, void *mod, unsigned long len);

struct module_options
{
	struct module_options *next;
	char *modulename;
	char *options;
};

struct module_command
{
	struct module_command *next;
	char *modulename;
	char *command;
};

struct module_alias
{
	struct module_alias *next;
	char *module;
};

struct module_blacklist
{
	struct module_blacklist *next;
	char *modulename;
};

struct module_options *
add_options(const char *modname,
	    const char *option,
	    struct module_options *options);
/* Link in a new install line from the config file. */
struct module_command *
add_command(const char *modname,
	       const char *command,
	    struct module_command *commands);

/* Link in a new alias line from the config file. */
struct module_alias *
add_alias(const char *modname, struct module_alias *aliases);

/* Link in a new blacklist line from the config file. */
struct module_blacklist *
add_blacklist(const char *modname, struct module_blacklist *blacklist);

/* Find blacklist commands if any. */
 int
find_blacklist(const char *modname, const struct module_blacklist *blacklist);

/* return a new alias list, with backlisted elems filtered out */
struct module_alias *
apply_blacklist(const struct module_alias *aliases,
		const struct module_blacklist *blacklist);

/* Find install commands if any. */
const char *find_command(const char *modname,
				const struct module_command *commands);

char *append_option(char *options, const char *newoption);

/* Add to options */
char *add_extra_options(const char *modname,
			       char *optstring,
			       const struct module_options *options);

/* If we don't flush, then child processes print before we do */
void verbose_printf(int verbose, const char *fmt, ...);
void do_command(const char *modname,
		       const char *command,
		       int verbose, int dry_run,
		       errfn_t error,
		       const char *type,
		       const char *cmdline_opts);

void insmod(struct list_head *list,
		   char *optstring,
		   const char *newname,
		   int first_time,
		   errfn_t error,
		   int dry_run,
		   int verbose,
		   const struct module_options *options,
		   const struct module_command *commands,
		   int ignore_commands,
		   int ignore_proc,
		   int strip_vermagic,
		   int strip_modversion,
		   const char *cmdline_opts);

void rmmod(struct list_head *list,
		  const char *name,
		  int first_time,
		  errfn_t error,
		  int dry_run,
		  int verbose,
		  struct module_command *commands,
		  int ignore_commands,
		  int ignore_inuse,
		  const char *cmdline_opts);

/* Does path contain directory(s) subpath? */
int type_matches(const char *path, const char *subpath);

/* Careful!  Don't munge - in [ ] as per Debian Bug#350915 */
char *underscores(char *string);

int do_wildcard(const char *dirname,
		       const char *type,
		       const char *wildcard);

char *strsep_skipspace(char **string, char *delim);
int read_aliases(const char *aliasfilename,
                        const char *name,
                        int removing,
                        struct module_options **options,
                        struct module_command **commands,
                        struct module_alias **aliases,
                        struct module_blacklist **blacklist);
int read_config(const char *filename,
		       const char *name,
		       int dump_only,
		       int removing,
			    struct module_options **options,
			    struct module_command **commands,
			    struct module_alias **aliases,
		       struct module_blacklist **blacklist);

int read_config_file(const char *filename,
			    const char *name,
			    int dump_only,
		       int removing,
		       struct module_options **options,
		       struct module_command **commands,
		       struct module_alias **aliases,
			    struct module_blacklist **blacklist);

int read_config(const char *filename,
		       const char *name,
		       int dump_only,
		       int removing,
				 struct module_options **options,
				 struct module_command **commands,
				 struct module_alias **aliases,
		       struct module_blacklist **blacklist);

void read_toplevel_config(const char *filename,
				 const char *name,
				 int dump_only,
				 int removing,
				 struct module_options **options,
				 struct module_command **commands,
				 struct module_alias **aliases,
				 struct module_blacklist **blacklist);

void add_to_env_var(const char *option);

/* Prepend options from environment. */
char **merge_args(char *args, char *argv[], int *argc);

char *gather_options(char *argv[]);

void handle_module(const char *modname,
			  struct list_head *todo_list,
			  const char *newname,
			  int remove,
			  char *options,
			  int first_time,
			  errfn_t error,
			  int dry_run,
			  int verbose,
			  struct module_options *modoptions,
			  struct module_command *commands,
			  int ignore_commands,
			  int ignore_proc,
			  int strip_vermagic,
			  int strip_modversion,
			  int unknown_silent,
			  const char *cmdline_opts);

/* This is a horrible hack to allow devfsd, which calls modprobe with
   -C /etc/modules.conf or /etc/modules.devfs, to work.  FIXME. */
/* Modern devfsd or variants should use -q explicitly in 2.6. */
int is_devfs_call(char *argv[]);


#define MODPROBE_DEVFSD_CONF "/etc/modprobe.devfs"
